﻿-- Macro Profiles
local MPS_Version = "2.0.1 Beta"

MPS = LibStub("AceAddon-3.0"):NewAddon("Profiles: Macros", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

local MAX_ACCOUNT_MACROS, MAX_CHARACTER_MACROS = 36, 18

--Char specific macros
minMacros = 1
charMacros = MAX_ACCOUNT_MACROS + 1
maxMacros = MAX_ACCOUNT_MACROS+MAX_CHARACTER_MACROS

function MPS:OnInitialize()
   MPdb = LibStub("AceDB-3.0"):New("ProfilesMacros",nil,"Default")
end

function MPS:Echo(text)
	if PROFILES_TYST then return nil end
   DEFAULT_CHAT_FRAME:AddMessage("|cFFFF0000Profiles: Macros - "..text)
   return 1
end

-- ProfilesHub: Send message to all slaves that we are now saving profile
function MPS:PreSaveMacroProfile(name)
	MPS:SendAceMsg("save",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:SaveMacroProfile(name)
	
	-- Save a restore point if empty db
	if not MPS:ExistsProfile("Restore [LOCKED]") and not MPSSAVERESTORE then 
		MPSSAVERESTORE=1 
		MPS:SaveMacroProfile("Restore [LOCKED]") 
		MPSSAVERESTORE=nil
	end
	
	-- User trying to overwrite restore point!
	if name == "Restore [LOCKED]" and not MPSSAVERESTORE then 
		MPS:Echo("This profile is locked at the first run and can only be loaded!")
		return nil
	end
	
	-- No macros!
	local numperglobal,numperchar = GetNumMacros()
	if numperchar == 0 and numperglobal == 0 then 
		MPS:Echo("Nothing to save!") 
		return nil 
	end
	
	-- Duplicate macro name!
	-- Since we are using the name to identify where its actionBar_pos is, WARN user
	
	MPdb:SetProfile(name)
	
	-- Solving /mp save tank X 2
	if MPS:ExistsProfile(name) then MPdb:ResetProfile(name) end
	
	MPS:Echo("Saving macros to profile "..name.."!")
	
	
	-- Cache icons path
	local icons_table = {}
	local numIcons = GetNumMacroIcons()
	for i=1,numIcons do
		local lowercase_icons = string.lower(GetMacroIconInfo(i)) -- Normalize icon path from the API
		icons_table[lowercase_icons] = i
	end
    
	-- Save macros
	for i=minMacros,maxMacros do
	   local macro_name, texture, body = GetMacroInfo(i)
	   local actionBar_pos = {};
	   
	   -- No more macros saved
	   if macro_name then 
		  
		  -- Get icon ID instead of path (needed for CreateMacro() in Load!
		  texture = string.lower(texture)
		  if icons_table[texture] then 
			 texture = icons_table[texture]
		  end
		  
		  -- Icon not found, give question mark!
		  if (type(texture) ~= "number") then 
			 texture = '1'  
		  end
		  
		  -- Save actionbar position
		  for actionID=1, 120 do
			 local _, id = GetActionInfo(actionID)
			 local abmacro_name, _, _ = GetMacroInfo(id)
			 
			 if i == id then
				table.insert(actionBar_pos , actionID)
			 end
		  end
		  
		  -- Save the GLOBAL macro to db
		  MPdb.profile[i] = { macro_name,texture,body,actionBar_pos,1,i}
	   end
	   
	end
		
	MPS:SendMessage("MPS_Profiles_save",name) -- ProfilesHub
	return name
end

-- This is an alias for ProfilesHub
function MPS:Save(...)
	MPS:SaveMacroProfile(...)
end


-- ProfilesHub: Send message to all slaves that we are now loading profile
function MPS:PreLoadMacroProfile(name)
	MPS:SendAceMsg("load",name) -- ProfilesHub
end

-- This function should NOT BE CALLED DIRECTLY.
-- Should only be called by slave event!

function MPS:LoadMacroProfile(name)
	
	-- Save a restore point if empty db
	if not MPS:ExistsProfile("Restore [LOCKED]") and not MPSSAVERESTORE then 
		MPSSAVERESTORE=1 
		MPS:SaveMacroProfile("Restore [LOCKED]") 
		MPSSAVERESTORE=nil
	end
	
	-- No such profile saved
	if not MPS:ExistsProfile(name) then MPS:Echo(name.." profile not found.") return nil end
	
	-- In combat! 
	if InCombatLockdown() then MPS:Echo("Cannot change macros in combat!") return nil end

   MPdb:SetProfile(name)
  
   MPS:Echo("Loading macros from profile "..name.."!")
   
	--Get rid of all the current macros
	for i=minMacros,MAX_ACCOUNT_MACROS do DeleteMacro(1)  end -- Global
	for i=charMacros,maxMacros do DeleteMacro(37) end -- Char
   
   --Load saved macros profile
   for index, keys in pairs(MPdb.profile) do   
       if keys[6] and index == keys[6] then
            if (type(keys[2]) ~= "number") then keys[2] = '1' end -- Icon not found, give question mark!
			
			if index >= charMacros then -- CreateMacro must know if its a global or char macro
				CreateMacro(keys[1], keys[2], keys[3], 1);
			else
				CreateMacro(keys[1], keys[2], keys[3], nil);
			end
       end
   end
   
   -- Action bar handling
   for index, keys in pairs(MPdb.profile) do   
       if keys[6] and index == keys[6] then	
			for _,v in pairs(keys[4]) do
				ClearCursor()
				
				-- Clear current spot
				PickupAction(v)
				ClearCursor()

				-- Restore this spot
				PickupMacro(keys[6])
				PlaceAction(v)
				ClearCursor()
			end
			
       end
   end
   
	if (IsAddOnLoaded("Blizzard_MacroUI")) then
		MacroFrame_Update() -- Blizzards own code
	end
   
	MPS:SendMessage("MPS_Profiles_load",name) -- ProfilesHub
	return name
end

-- This is an alias for ProfilesHub
function MPS:Load(...)
	MPS:LoadMacroProfile(...)
end

function MPS:ExistsProfile(name)

	local callingName = nil
	
	for i, k in ipairs(MPdb:GetProfiles()) do
	
		if k == name then
			callingName = name
		end
	end
   
	if callingName then 
		return callingName
	else	
		return nil
   end
end

function MPS:ListProfiles()
    MPS:Echo(" Available Profiles: ");
    
	for i,k in ipairs(MPdb:GetProfiles()) do
		if k == MPdb:GetCurrentProfile() then
			MPS:Echo(' ' .. k.." (current)")
		else
			MPS:Echo(' ' .. k)
		end
	end
	
	return 1
end

-- ProfilesHub: Send message to all slaves that we are now deleting a profile
function MPS:PreDeleteProfile(name)
	MPS:SendAceMsg("delete",name)
end

function MPS:DeleteProfil(name)
	if name == "Restore [LOCKED]" and not MPSSAVERESTORE then 
		MPS:Echo("This profile cannot be deleted! It is locked at the first run and can only be loaded!")
		return nil
	end
	
	if (MPS:ExistsProfile(name)==nil)then
		MPS:Echo(name.." doesnt exist.")
		MPS:ListProfiles()
		
		return nil
	else
		if name == MPdb:GetCurrentProfile() then
			MPdb:SetProfile('Default')
		end
		MPdb:DeleteProfile(name)
		MPS:Echo(name.." deleted.")
		
		return name
	end
end

-- This is an alias for ProfilesHub
function MPS:Delete(...)
	KPS:DeleteProfil(...)
end